@@ -1,23 +1,14 @@
 /*
  * This file is part of the flashrom project.
  *
- * Copyright (C) 2000 Silicon Integrated System Corporation
- * Copyright (C) 2005-2009 coresystems GmbH
- * Copyright (C) 2006 Uwe Hermann <uwe@hermann-uwe.de>
- * Copyright (C) 2007,2008,2009 Carl-Daniel Hailfinger
- * Copyright (C) 2009 Kontron Modular Computers GmbH
- * Copyright (C) 2011, 2012 Stefan Tauner
- * Copyright (C) 2017 secunet Security Networks AG
- * (Written by Nico Huber <nico.huber@secunet.com> for secunet)
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * SPDX-License-Identifier: GPL-2.0-only
+ * SPDX-FileCopyrightText: 2000 Silicon Integrated System Corporation
+ * SPDX-FileCopyrightText: 2005-2009 coresystems GmbH
+ * SPDX-FileCopyrightText: 2006 Uwe Hermann <uwe@hermann-uwe.de>
+ * SPDX-FileCopyrightText: 2007,2008,2009 Carl-Daniel Hailfinger
+ * SPDX-FileCopyrightText: 2009 Kontron Modular Computers GmbH
+ * SPDX-FileCopyrightText: 2011, 2012 Stefan Tauner
+ * SPDX-FileCopyrightText: 2017 secunet Security Networks AG (Written by Nico Huber <nico.huber@secunet.com> for secunet)
  */
 
 /*
@@ -33,7 +24,7 @@
 #include "flash.h"
 #include "programmer.h"
 #include "hwaccess_physmap.h"
-#include "platform/pci.h"
+#include "pcidev.h"
 
 #define NOT_DONE_YET 1
 
@@ -609,6 +600,7 @@ static enum chipbustype enable_flash_ich_report_gcs(
 	case CHIPSET_700_SERIES_RAPTOR_POINT:
 	case CHIPSET_METEOR_LAKE:
 	case CHIPSET_PANTHER_LAKE:
+	case CHIPSET_WILDCAT_LAKE:
 	case CHIPSET_ELKHART_LAKE:
 	case CHIPSET_APOLLO_LAKE:
 	case CHIPSET_GEMINI_LAKE:
@@ -719,6 +711,7 @@ static enum chipbustype enable_flash_ich_report_gcs(
 	case CHIPSET_C740_SERIES_EMMITSBURG:
 	case CHIPSET_METEOR_LAKE:
 	case CHIPSET_PANTHER_LAKE:
+	case CHIPSET_WILDCAT_LAKE:
 		boot_straps = boot_straps_pch500;
 		break;
 	case CHIPSET_APOLLO_LAKE:
@@ -756,6 +749,7 @@ static enum chipbustype enable_flash_ich_report_gcs(
 	case CHIPSET_700_SERIES_RAPTOR_POINT:
 	case CHIPSET_METEOR_LAKE:
 	case CHIPSET_PANTHER_LAKE:
+	case CHIPSET_WILDCAT_LAKE:
 	case CHIPSET_APOLLO_LAKE:
 	case CHIPSET_GEMINI_LAKE:
 	case CHIPSET_JASPER_LAKE:
@@ -1041,6 +1035,11 @@ static int enable_flash_ptl(const struct programmer_cfg *cfg, struct pci_dev *co
 	return enable_flash_pch100_or_c620(cfg, dev, name, 0x1f, 5, CHIPSET_PANTHER_LAKE);
 }
 
+static int enable_flash_wcl(const struct programmer_cfg *cfg, struct pci_dev *const dev, const char *const name)
+{
+        return enable_flash_pch100_or_c620(cfg, dev, name, 0x1f, 5, CHIPSET_WILDCAT_LAKE);
+}
+
 static int enable_flash_mcc(const struct programmer_cfg *cfg, struct pci_dev *const dev, const char *const name)
 {
 	return enable_flash_pch100_or_c620(cfg, dev, name, 0x1f, 5, CHIPSET_ELKHART_LAKE);
@@ -2220,8 +2219,10 @@ const struct penable chipset_enables[] = {
 	{0x8086, 0x7a14, B_S,    NT,  "Intel", "C262",				enable_flash_pch700},
 	{0x8086, 0x7a13, B_S,    NT,  "Intel", "C266",				enable_flash_pch700},
 	{0x8086, 0x7e23, B_S,    DEP, "Intel", "Meteor Lake-P/M",		enable_flash_mtl},
+	{0x8086, 0x7723, B_S,    DEP, "Intel", "Arrow Lake-H",			enable_flash_mtl},
 	{0x8086, 0xe323, B_S,    DEP, "Intel", "Panther Lake-U/H 12Xe",		enable_flash_ptl},
 	{0x8086, 0xe423, B_S,    DEP, "Intel", "Panther Lake-H 4Xe",		enable_flash_ptl},
+	{0x8086, 0x4d23, B_S,    DEP, "Intel", "Wildcat Lake",                  enable_flash_wcl},
 #endif
 	{0},
 };
