@@ -1,20 +1,11 @@
 /*
  * This file is part of the flashrom project.
  *
- * Copyright (C) 2000 Silicon Integrated System Corporation
- * Copyright (C) 2004 Tyan Corp <yhlu@tyan.com>
- * Copyright (C) 2005-2008 coresystems GmbH
- * Copyright (C) 2008,2009,2010 Carl-Daniel Hailfinger
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-FileCopyrightText: 2000 Silicon Integrated System Corporation
+ * SPDX-FileCopyrightText: 2004 Tyan Corp <yhlu@tyan.com>
+ * SPDX-FileCopyrightText: 2005-2008 coresystems GmbH
+ * SPDX-FileCopyrightText: 2008,2009,2010 Carl-Daniel Hailfinger
  */
 
 #include <errno.h>
@@ -27,6 +18,7 @@
 #include <stdint.h>
 #include <cli_getopt.h>
 #include <cli_output.h>
+#include <time.h>
 #include "flash.h"
 #include "flashchips.h"
 #include "fmap.h"
@@ -41,6 +33,7 @@ enum {
 	OPTION_IFD = 0x0100,
 	OPTION_FMAP,
 	OPTION_FMAP_FILE,
+	OPTION_FMAP_VERIFY,
 	OPTION_FLASH_CONTENTS,
 	OPTION_FLASH_NAME,
 	OPTION_FLASH_SIZE,
@@ -73,7 +66,7 @@ struct cli_options {
 	const struct programmer_entry *prog;
 	char *pparam;
 
-	bool ifd, fmap;
+	bool ifd, fmap, fmap_verify;
 	struct flashrom_layout *layout;
 	struct layout_include_args *include_args;
 	char *layoutfile;
@@ -140,6 +133,7 @@ static void cli_classic_usage(const char *name)
 	       "      --flash-size                  read out the detected flash size\n"
 	       "      --fmap                        read ROM layout from fmap embedded in ROM\n"
 	       "      --fmap-file <fmapfile>        read ROM layout from fmap in <fmapfile>\n"
+	       "      --fmap-verify                 read ROM layout from fmap and verify it matches file fmap\n"
 	       "      --ifd                         read layout from an Intel Firmware Descriptor\n"
 	       " -i | --include <region>[:<file>]   only read/write image <region> from layout\n"
 	       "                                    (optionally with data from <file>)\n"
@@ -855,7 +849,7 @@ static void parse_options(int argc, char **argv, const char *optstring,
 			break;
 		case OPTION_FMAP_FILE:
 			if (options->fmap)
-				cli_classic_abort_usage("Error: --fmap or --fmap-file specified "
+				cli_classic_abort_usage("Error: --fmap, --fmap-file, or --fmap-verify specified "
 					"more than once. Aborting.\n");
 			if (options->ifd)
 				cli_classic_abort_usage("Error: --fmap-file and --ifd both specified. Aborting.\n");
@@ -866,7 +860,7 @@ static void parse_options(int argc, char **argv, const char *optstring,
 			break;
 		case OPTION_FMAP:
 			if (options->fmap)
-				cli_classic_abort_usage("Error: --fmap or --fmap-file specified "
+				cli_classic_abort_usage("Error: --fmap, --fmap-file, or --fmap-verify specified "
 					"more than once. Aborting.\n");
 			if (options->ifd)
 				cli_classic_abort_usage("Error: --fmap and --ifd both specified. Aborting.\n");
@@ -874,6 +868,19 @@ static void parse_options(int argc, char **argv, const char *optstring,
 				cli_classic_abort_usage("Error: --layout and --fmap both specified. Aborting.\n");
 			options->fmap = true;
 			break;
+		case OPTION_FMAP_VERIFY:
+			if (options->fmap)
+				cli_classic_abort_usage("Error: --fmap, --fmap-file, or --fmap-verify specified "
+					"more than once. Aborting.\n");
+			if (options->ifd)
+				cli_classic_abort_usage("Error: --fmap-verify and --ifd both specified. Aborting.\n");
+			if (options->layoutfile)
+				cli_classic_abort_usage("Error: --fmap-verify and --layout both specified. Aborting.\n");
+			if (options->read_it || options->verify_it)
+				cli_classic_abort_usage("Error: --fmap-verify cannot be used with read or verify operations. Aborting.\n");
+			options->fmap = true;
+			options->fmap_verify = true;
+			break;
 		case 'i':
 			if (register_include_arg(&options->include_args, optarg))
 				cli_classic_abort_usage(NULL);
@@ -1043,12 +1050,19 @@ static void free_options(struct cli_options *options)
 int main(int argc, char *argv[])
 {
 	const struct flashchip *chip = NULL;
-	struct flashctx context = {0}; /* holds the active detected chip and other info */
 	char *tempstr = NULL;
 	int i, j;
 	int ret = 0;
 	int all_matched_count = 0;
 	const char **all_matched_names = NULL;
+	time_t time_start, time_end;
+
+	struct flashctx *context = NULL; /* holds the active detected chip and other info */
+	ret = flashrom_create_context(&context);
+	if (ret) {
+		msg_gerr("Failed to allocate flash context. Aborting");
+		return 1;
+	}
 
 	struct cli_options options = { 0 };
 	static const char optstring[] = "r::Rw::v::nNVEfc:l:i:p:Lzho:x";
@@ -1067,6 +1081,7 @@ int main(int argc, char *argv[])
 		{"ifd",			0, NULL, OPTION_IFD},
 		{"fmap",		0, NULL, OPTION_FMAP},
 		{"fmap-file",		1, NULL, OPTION_FMAP_FILE},
+		{"fmap-verify",		0, NULL, OPTION_FMAP_VERIFY},
 		{"image",		1, NULL, 'i'}, // (deprecated): back compatibility.
 		{"include",		1, NULL, 'i'},
 		{"flash-contents",	1, NULL, OPTION_FLASH_CONTENTS},
@@ -1196,6 +1211,8 @@ int main(int argc, char *argv[])
 	if (flashrom_init(1))
 		exit(1);
 
+	time(&time_start);
+
 	if (programmer_init(options.prog, options.pparam)) {
 		msg_perr("Error: Programmer initialization failed.\n");
 		ret = 1;
@@ -1205,7 +1222,7 @@ int main(int argc, char *argv[])
 	msg_pdbg("The following protocols are supported: %s.\n", tempstr ? tempstr : "?");
 	free(tempstr);
 
-	all_matched_count = flashrom_flash_probe_v2(&context, &all_matched_names,
+	all_matched_count = flashrom_flash_probe_v2(context, &all_matched_names,
                                 NULL, options.chip_to_probe);
 	if (all_matched_count == -1) {
 		/* -1 is the ret code which means "something went wrong".
@@ -1218,7 +1235,7 @@ int main(int argc, char *argv[])
 
 	if (all_matched_count > 1) {
 		msg_cinfo("Multiple flash chip definitions match the detected chip(s): \"%s\"",
-			  context.chip->name);
+			  context->chip->name);
 		for (int ind = 1; ind < all_matched_count; ind++)
 			msg_cinfo(", \"%s\"", all_matched_names[ind]);
 		msg_cinfo("\nPlease specify which chip definition to use with the -c <chipname> option.\n");
@@ -1253,7 +1270,7 @@ int main(int argc, char *argv[])
 			int force_probe_ret = ERROR_FLASHROM_PROBE_NO_CHIPS_FOUND;
 			for (j = 0; j < registered_master_count; j++) {
 				mst = &registered_masters[j];
-				force_probe_ret = probe_flash(mst, 0, &context, 1, options.chip_to_probe);
+				force_probe_ret = probe_flash(mst, 0, context, 1, options.chip_to_probe);
 				if (force_probe_ret >= 0)
 					break;
 			}
@@ -1264,31 +1281,31 @@ int main(int argc, char *argv[])
 				goto out_shutdown;
 			}
 			msg_cinfo("Please note that forced reads most likely contain garbage.\n");
-			flashrom_flag_set(&context, FLASHROM_FLAG_FORCE, options.force);
-			ret = do_read(&context, options.filename);
-			free(context.chip);
+			flashrom_flag_set(context, FLASHROM_FLAG_FORCE, options.force);
+			ret = do_read(context, options.filename);
+			free(context->chip);
 			goto out_shutdown;
 		}
 		ret = 1;
 		goto out_shutdown;
 	} else if (!options.chip_to_probe) {
 		/* repeat for convenience when looking at foreign logs */
-		tempstr = flashbuses_to_text(context.chip->bustype);
+		tempstr = flashbuses_to_text(context->chip->bustype);
 		msg_gdbg("Found %s flash chip \"%s\" (%d kB, %s).\n",
-			 context.chip->vendor, context.chip->name, context.chip->total_size,
+			 context->chip->vendor, context->chip->name, context->chip->total_size,
 			 tempstr ? tempstr : "?");
 		free(tempstr);
 	}
 
 	struct cli_progress cli_progress = {0};
 	if (options.show_progress)
-		flashrom_set_progress_callback_v2(&context, &flashrom_progress_cb, &cli_progress);
+		flashrom_set_progress_callback_v2(context, &flashrom_progress_cb, &cli_progress);
 
-	print_chip_support_status(context.chip);
+	print_chip_support_status(context->chip);
 
-	unsigned int limitexceeded = count_max_decode_exceedings(&context, &max_rom_decode);
+	unsigned int limitexceeded = count_max_decode_exceedings(context, &max_rom_decode);
 	if (limitexceeded > 0 && !options.force) {
-		enum chipbustype commonbuses = context.mst->buses_supported & context.chip->bustype;
+		enum chipbustype commonbuses = context->mst->buses_supported & context->chip->bustype;
 
 		/* Sometimes chip and programmer have more than one bus in common,
 		 * and the limit is not exceeded on all buses. Tell the user. */
@@ -1386,10 +1403,10 @@ int main(int argc, char *argv[])
 	}
 
 	if (options.flash_name) {
-		if (context.chip->vendor && context.chip->name) {
+		if (context->chip->vendor && context->chip->name) {
 			printf("vendor=\"%s\" name=\"%s\"\n",
-				context.chip->vendor,
-				context.chip->name);
+				context->chip->vendor,
+				context->chip->name);
 		} else {
 			ret = -1;
 		}
@@ -1397,7 +1414,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (options.flash_size) {
-		printf("%zu\n", flashrom_flash_getsize(&context));
+		printf("%zu\n", flashrom_flash_getsize(context));
 		goto out_shutdown;
 	}
 
@@ -1406,10 +1423,10 @@ int main(int argc, char *argv[])
 			msg_ginfo("Invalid input of sacrifice ratio, valid 0-50. Fallback to default value 0.\n");
 			options.sacrifice_ratio = 0;
 		}
-		context.sacrifice_ratio = options.sacrifice_ratio;
+		context->sacrifice_ratio = options.sacrifice_ratio;
 	}
 
-	if (options.ifd && (flashrom_layout_read_from_ifd(&options.layout, &context, NULL, 0) ||
+	if (options.ifd && (flashrom_layout_read_from_ifd(&options.layout, context, NULL, 0) ||
 			   process_include_args(options.layout, options.include_args))) {
 		ret = 1;
 		goto out_shutdown;
@@ -1434,20 +1451,63 @@ int main(int argc, char *argv[])
 			goto out_shutdown;
 		}
 
-		if (flashrom_layout_read_fmap_from_buffer(&options.layout, &context, fmapfile_buffer, fmapfile_size) ||
+		if (flashrom_layout_read_fmap_from_buffer(&options.layout, context, fmapfile_buffer, fmapfile_size) ||
 		    process_include_args(options.layout, options.include_args)) {
 			ret = 1;
 			free(fmapfile_buffer);
 			goto out_shutdown;
 		}
 		free(fmapfile_buffer);
-	} else if (options.fmap && (flashrom_layout_read_fmap_from_rom(&options.layout, &context, 0,
-				flashrom_flash_getsize(&context)) ||
-				process_include_args(options.layout, options.include_args))) {
-		ret = 1;
-		goto out_shutdown;
+	} else if (options.fmap) {
+		/* Read layout from ROM fmap */
+		if (flashrom_layout_read_fmap_from_rom(&options.layout, context, 0,
+				flashrom_flash_getsize(context)) ||
+				process_include_args(options.layout, options.include_args)) {
+			ret = 1;
+			goto out_shutdown;
+		}
+		if (options.fmap_verify) {
+			struct flashrom_layout *file_layout = NULL;
+			struct stat s;
+			if (stat(options.filename, &s) != 0) {
+				msg_gerr("Failed to stat the file \"%s\"\n", options.filename);
+				ret = 1;
+				goto out_release;
+			}
+
+			size_t fmapfile_size = s.st_size;
+			uint8_t *file_buffer = malloc(fmapfile_size);
+			if (!file_buffer) {
+				ret = 1;
+				goto out_release;
+			}
+
+			if (read_buf_from_file(file_buffer, fmapfile_size, options.filename)) {
+				ret = 1;
+				free(file_buffer);
+				goto out_release;
+			}
+			/* Read layout from file fmap */
+			if (flashrom_layout_read_fmap_from_buffer(&file_layout, context, file_buffer,
+					flashrom_flash_getsize(context)) ||
+					process_include_args(file_layout, options.include_args)) {
+				ret = 1;
+				free(file_buffer);
+				goto out_release;
+			}
+			free(file_buffer);
+			/* compare the two layouts */
+			if (flashrom_layout_compare(options.layout, file_layout)) {
+				msg_cerr("FMAP layouts do not match! Aborting.\n");
+				flashrom_layout_release(file_layout);
+				ret = 1;
+				goto out_release;
+			}
+			flashrom_layout_release(file_layout);
+			msg_cinfo("FMAP layouts match.\n");
+		}
 	}
-	flashrom_layout_set(&context, options.layout);
+	flashrom_layout_set(context, options.layout);
 
 	if (any_wp_op) {
 		if (options.set_wp_region && options.wp_region) {
@@ -1465,7 +1525,7 @@ int main(int argc, char *argv[])
 			options.set_wp_range = true;
 		}
 		ret = wp_cli(
-			&context,
+			context,
 			options.enable_wp,
 			options.disable_wp,
 			options.print_wp_status,
@@ -1478,42 +1538,33 @@ int main(int argc, char *argv[])
 			goto out_release;
 	}
 
-	flashrom_flag_set(&context, FLASHROM_FLAG_FORCE, options.force);
+	flashrom_flag_set(context, FLASHROM_FLAG_FORCE, options.force);
 #if CONFIG_INTERNAL == 1
-	flashrom_flag_set(&context, FLASHROM_FLAG_FORCE_BOARDMISMATCH, force_boardmismatch);
+	flashrom_flag_set(context, FLASHROM_FLAG_FORCE_BOARDMISMATCH, force_boardmismatch);
 #endif
-	flashrom_flag_set(&context, FLASHROM_FLAG_VERIFY_AFTER_WRITE, !options.dont_verify_it);
-	flashrom_flag_set(&context, FLASHROM_FLAG_VERIFY_WHOLE_CHIP, !options.dont_verify_all);
+	flashrom_flag_set(context, FLASHROM_FLAG_VERIFY_AFTER_WRITE, !options.dont_verify_it);
+	flashrom_flag_set(context, FLASHROM_FLAG_VERIFY_WHOLE_CHIP, !options.dont_verify_all);
 
 	/* FIXME: We should issue an unconditional chip reset here. This can be
 	 * done once we have a .reset function in struct flashchip.
 	 * Give the chip time to settle.
 	 */
-	programmer_delay(&context, 100000);
+	programmer_delay(context, 100000);
 	if (options.read_it)
-		ret = do_read(&context, options.filename);
+		ret = do_read(context, options.filename);
 	else if (options.extract_it)
-		ret = do_extract(&context);
+		ret = do_extract(context);
 	else if (options.erase_it) {
-		ret = flashrom_flash_erase(&context);
-		/*
-		 * FIXME: Do we really want the scary warning if erase failed?
-		 * After all, after erase the chip is either blank or partially
-		 * blank or it has the old contents. A blank chip won't boot,
-		 * so if the user wanted erase and reboots afterwards, the user
-		 * knows very well that booting won't work.
-		 */
-		if (ret)
-			emergency_help_message();
+		ret = flashrom_flash_erase(context);
 	}
 	else if (options.write_it)
-		ret = do_write(&context, options.filename, options.referencefile);
+		ret = do_write(context, options.filename, options.referencefile);
 	else if (options.verify_it)
-		ret = do_verify(&context, options.filename);
+		ret = do_verify(context, options.filename);
 
 #if CONFIG_RPMC_ENABLED == 1
 	if (any_rpmc_op && ret == 0) {
-		ret = rpmc_cli(&context,
+		ret = rpmc_cli(context,
 			       options.rpmc_root_key_file,
 			       options.rpmc_key_data,
 			       options.rpmc_counter_address,
@@ -1531,11 +1582,15 @@ out_release:
 out_shutdown:
 	flashrom_programmer_shutdown(NULL);
 out:
-	flashrom_layout_release(context.default_layout);
-	free(context.chip);
 	flashrom_data_free(all_matched_names);
+	flashrom_flash_release(context);
 
 	free_options(&options);
+
+	time(&time_end);
+	msg_gdbg("Runtime from programmer init to shutdown: %dmin%2dsec\n",
+		(int)(difftime(time_end, time_start) / 60), (int)(difftime(time_end, time_start)) % 60);
+
 	ret |= close_logfile();
 	return ret;
 }
