@@ -1,20 +1,10 @@
 /*
  * This file is part of the flashrom project.
  *
- * Copyright (C) 2012, 2016 secunet Security Networks AG
- * (Written by Nico Huber <nico.huber@secunet.com> for secunet)
- * Copyright (C) 2025 Dmitry Zhadinets <dzhadinets@gmail.com>
- * Copyright 2025 Google LLC
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-FileCopyrightText: 2012, 2016 secunet Security Networks AG (Written by Nico Huber <nico.huber@secunet.com> for secunet)
+ * SPDX-FileCopyrightText: 2025 Dmitry Zhadinets <dzhadinets@gmail.com>
+ * SPDX-FileCopyrightText: 2025 Google LLC
  */
 
 #include <errno.h>
@@ -49,6 +39,23 @@ int flashrom_shutdown(void)
 	return 0; /* TODO: nothing to do? */
 }
 
+int flashrom_create_context(struct flashrom_flashctx **const flashctx)
+{
+	struct flashrom_flashctx *const out = malloc(sizeof(struct flashrom_flashctx));
+	if (!out)
+		return 1;
+
+	/* Init of values inside context. */
+	*out = (struct flashrom_flashctx) {
+		.virtual_memory = (chipaddr)ERROR_PTR,
+		.virtual_registers = (chipaddr)ERROR_PTR,
+	};
+
+	*flashctx = out;
+
+	return 0;
+}
+
 void flashrom_set_log_level(enum flashrom_log_level level)
 {
 	global_log_level = level;
@@ -455,15 +462,54 @@ bool flashrom_flag_get(const struct flashrom_flashctx *const flashctx, const enu
 	}
 }
 
+#ifdef __FLASHROM_LITTLE_ENDIAN__
 static int compare_region_with_dump(const struct romentry *const a, const struct romentry *const b)
 {
-	if (a->region.start != b->region.end
+	if (a->region.start != b->region.start
 		|| a->region.end != b->region.end
 		|| strcmp(a->region.name, b->region.name))
 			return 1;
 	return 0;
 }
 
+int flashrom_layout_compare(const struct flashrom_layout *layout1,
+			    const struct flashrom_layout *layout2)
+{
+	if (!layout1 || !layout2) {
+		msg_gerr("Error: NULL layout pointer in comparison.\n");
+		return 1;
+	}
+
+	const struct romentry *entry1 = layout_next(layout1, NULL);
+	const struct romentry *entry2 = layout_next(layout2, NULL);
+
+	while (entry1 && entry2) {
+		if (compare_region_with_dump(entry1, entry2)) {
+			msg_gerr("Layout region mismatch:\n");
+			msg_gerr("  Region 1: '%s' [0x%08" PRIx32 ":0x%08" PRIx32 "]\n",
+				entry1->region.name, entry1->region.start, entry1->region.end);
+			msg_gerr("  Region 2: '%s' [0x%08" PRIx32 ":0x%08" PRIx32 "]\n",
+				entry2->region.name, entry2->region.start, entry2->region.end);
+			return 1;
+		}
+		entry1 = layout_next(layout1, entry1);
+		entry2 = layout_next(layout2, entry2);
+	}
+
+	/* If one layout has more regions than the other */
+	if (entry1 || entry2) {
+		msg_gerr("Layout region count mismatch.\n");
+		if (entry1)
+			msg_gerr("  Extra region in layout 1: '%s'\n", entry1->region.name);
+		if (entry2)
+			msg_gerr("  Extra region in layout 2: '%s'\n", entry2->region.name);
+		return 1;
+	}
+
+	return 0;
+}
+#endif /* __FLASHROM_LITTLE_ENDIAN__ */
+
 int flashrom_layout_read_from_ifd(struct flashrom_layout **const layout, struct flashctx *const flashctx,
 				  const void *const dump, const size_t len)
 {
@@ -523,7 +569,7 @@ _free_ret:
 		flashrom_layout_release(chip_layout);
 	free(desc);
 	return ret;
-#endif
+#endif /* __FLASHROM_LITTLE_ENDIAN__ */
 }
 
 #ifdef __FLASHROM_LITTLE_ENDIAN__
